package Plugins::WebService;

use Config::General;
use Encode;

my $conf = new Config::General(
    -ConfigFile      => $config::plugins_dir . '/WebService.conf',
    -InterPolateVars => 'TRUE',
    -AutoLaunder     => 'TRUE'
);
my %cfg = $conf->getall;

my $VERSION  = 1.1;
my $this_pkg = (caller(0))[0];

##################################################################
#   beforeAddUser
##################################################################
sub beforeAddUser {
    my ($status_hash, $ldap, $base_dn, $entry_in) = @_;
    my ($this_fun, $msg, $code, $out_panic);
    $this_fun  = (caller(0))[3];
    $msg       = '';
    $code      = 0;
    $out_panic = 0;

    AOSICFG::write_ws_log("|> $this_fun(in): ldap=$ldap;\nbase_dn=$base_dn;\nentry_in=\n" . Plugins::dump_entry($entry_in)) if $config::aosi_debug_level > 1;
    ($code, $msg) = sendToWS('beforeAddUser', $entry_in);
    if ($code != 0) {
        $out_panic = 1;
    }
    AOSICFG::write_ws_log("|> $this_fun(out): code=$code;msg=$msg;panic=$out_panic;") if $config::aosi_debug_level > 1;
    return $code, $msg, $out_panic;
}

##################################################################
#   afterAddUser
##################################################################
sub afterAddUser {
    my ($status_hash, $entry_in, $out_xml) = @_;
    my ($this_fun, $msg, $code, $out_panic);
    $this_fun  = (caller(0))[3];
    $msg       = '';
    $code      = 0;
    $out_panic = 0;
    AOSICFG::write_ws_log("|> $this_fun(in): \nentry_in=\n" . Plugins::dump_entry($entry_in) . "\nout_xml=\n" . $out_xml) if $config::aosi_debug_level > 1;
    ($code, $msg) = Plugins::parse_ldap_xml($out_xml);

    if ($code == 0) {
        ($code, $msg) = sendToWS('afterAddUser', $entry_in);
    }
    AOSICFG::write_ws_log("|> $this_fun(out): code=$code;msg=$msg;panic=$out_panic;") if $config::aosi_debug_level > 1;
    return $code, $msg, $out_panic;
}

##################################################################
#   beforeDeleteUser
##################################################################
sub beforeDeleteUser {
    my ($status_hash, $ldap, $base_dn, $entry_in) = @_;
    my ($this_fun, $msg, $dn, $code, $file, $panic, $out_panic);
    $this_fun  = (caller(0))[3];
    $msg       = '';
    $code      = 0;
    $out_panic = 0;
    AOSICFG::write_ws_log("|> $this_fun(in): \nentry_in=\n" . Plugins::dump_entry($entry_in)) if $config::aosi_debug_level > 1;
    ($code, $msg) = sendToWS('beforeDeleteUser', $entry_in);

    if ($code != 0) {
        $out_panic = 1;
    }
    AOSICFG::write_ws_log("|> $this_fun(out): code=$code;msg=$msg;panic=$out_panic;") if $config::aosi_debug_level > 1;
    return $code, $msg, $out_panic;
}

##################################################################
#   afterDeleteUser
##################################################################
sub afterDeleteUser {
    my ($status_hash, $entry_in, $out_xml) = @_;
    my ($this_fun, $msg, $code, $out_panic);
    $this_fun  = (caller(0))[3];
    $msg       = '';
    $code      = 0;
    $out_panic = 0;
    AOSICFG::write_ws_log("|> $this_fun(in): entry_in=" . Plugins::dump_entry($entry_in) . ";out_xml=" . $out_xml) if $config::aosi_debug_level > 1;
    ($code, $msg) = Plugins::parse_ldap_xml($out_xml);

    if ($code == 0) {
        ($code, $msg) = sendToWS('afterDeleteUser', $entry_in);
    }
    AOSICFG::write_ws_log("|> $this_fun(out): code=$code;msg=$msg;panic=$out_panic;") if $config::aosi_debug_level > 1;
    return $code, $msg, $out_panic;
}

##################################################################
#   before*Attribute
##################################################################
# Same function for all actions
# 1 = beforeAddAttribute
# 2 = beforeModifyAttribute
# 3 = beforeDeleteAttribute
sub beforeChangeAttribute {
    my ($status_hash, $type, $ldap, $base_dn, $entry_in) = @_;
    my ($this_fun, $msg, $code, $file, $out_panic);
    $this_fun  = (caller(0))[3];
    $msg       = '';
    $code      = 0;
    $out_panic = 0;

    AOSICFG::write_ws_log("|> $this_fun(in): ldap=$ldap;base_dn=$base_dn;entry_in=" . Plugins::dump_entry($entry_in)) if $config::aosi_debug_level > 1;
    ($code, $msg) = sendToWS('beforeChangeAttribute', $entry_in, $type);
    if ($code != 0) {
        $out_panic = 1;
    }
    AOSICFG::write_ws_log("|> $this_fun(out): code=$code;msg=$msg;panic=$out_panic;") if $config::aosi_debug_level > 1;
    return $code, $msg, $out_panic;
}

##################################################################
#   after*Attribute
##################################################################
# Same function for all actions
# 1 = afterAddAttribute
# 2 = afterModifyAttribute
# 3 = afterDeleteAttribute
sub afterChangeAttribute {
    my ($status_hash, $type, $entry_in, $out_xml) = @_;
    my ($this_fun, $code, $msg, $out_panic);
    $this_fun  = (caller(0))[3];
    $msg       = '';
    $code      = 0;
    $out_panic = 0;

    AOSICFG::write_ws_log("|> $this_fun(in): entry_in=" . Plugins::dump_entry($entry_in) . ";out_xml=" . $out_xml) if $config::aosi_debug_level > 1;
    ($code, $msg) = Plugins::parse_ldap_xml($out_xml);
    if ($code == 0) {
        ($code, $msg) = sendToWS('afterChangeAttribute', $entry_in, $type);
    }
    AOSICFG::write_ws_log("|> $this_fun(out): code=$code;msg=$msg;panic=$out_panic;") if $config::aosi_debug_level > 1;
    return $code, $msg, $out_panic;
}

sub sendToWS {
    my ($action, $entry, $change_type) = @_;
    my ($this_fun, $s, $soap_error, $soap_ver, $sd_data, $eval_stg, $code, $msg, $cur_code, $cur_msg);
    my ($svc, $svc_fun, $svc_loc, $svc_code, $svc_msg, $svc_ns, $result, $stg_entry, @params, $stg_entry_enc);
    $this_fun = (caller(0))[3];
    $soap_ver = $SOAP::Lite::VERSION;
    $msg      = '';
    $code     = 0;
    $change_type = $change_type eq '' ? 0 : $change_type;

    AOSICFG::write_ws_log("|> $this_fun(in): action=$action;type=$change_type;entry=" . Plugins::dump_entry($entry)) if $config::aosi_debug_level > 1;

    foreach $svc (keys %{ $cfg{'service'} }) {
        $svc_loc = $cfg{'service'}->{$svc}->{'location'};
        $svc_ns  = $cfg{'service'}->{$svc}->{'namespace'};
        $svc_fun = $cfg{'service'}->{$svc}->{'functions'}->{$action};
        AOSICFG::write_ws_log("|> $this_fun(service): name=$svc;location=$svc_loc;ns=$svc_ns;function=$svc_fun") if $config::aosi_debug_level > 2;
        $stg_entry = Plugins::dump_entry($entry);
        AOSICFG::write_ws_log("|> $this_fun: generating entry") if $config::aosi_debug_level > 2;
		    $stg_entry_enc = MIME::Base64::encode_base64(encode("UTF-8",$stg_entry));
        $sd_data   = SOAP::Data->name('data')->type('base64')->value($stg_entry_enc);
        AOSICFG::write_ws_log("|> $this_fun: preparing entry for SOAP") if $config::aosi_debug_level > 2;
        if (($svc_loc ne '') and ($svc_ns ne '')) {
            $s = SOAP::Lite->uri($svc_ns)->proxy($svc_loc);
            $s->{_transport}->{_proxy}->{ssl_opts}->{SSL_verify_mode} = 'SSL_VERIFY_NONE';
            AOSICFG::write_ws_log("|> $this_fun: SOAP object created") if $config::aosi_debug_level > 2;
            $s->on_fault(
                sub {
                    my ($soap, $res) = @_;
                    $soap_error = ref $res ? $res->faultstring : $soap->transport->status;
                }
            );
            AOSICFG::write_ws_log("|> $this_fun: soap_error=$soap_error") if $config::aosi_debug_level > 2;
            if ($soap_error eq '') {
                AOSICFG::write_ws_log("|> $this_fun: no SOAP error") if $config::aosi_debug_level > 2;
                if ($svc_fun ne '') {
                    AOSICFG::write_ws_log("|> $this_fun: calling function") if $config::aosi_debug_level > 2;
                    $result = $s->call($svc_fun, ($action, $sd_data, $change_type));
                    AOSICFG::write_ws_log("|> $this_fun: result=".$result) if $config::aosi_debug_level > 2;
                    if($result =~ /^500/){
                        $cur_msg = $result || $soap_error;
                        $cur_code = -10304;
                    }
                    else {
                        AOSICFG::write_ws_log("|> $this_fun: processing result: ".$result) if $config::aosi_debug_level > 2;
                        $cur_msg    = $result->valueof('//message');
                        $cur_code   = $result->valueof('//code');
                        Encode::_utf8_on($cur_msg);
                    }
                }
                else {
                    $cur_msg  = 'Service function not specified in configuration for action ' . $action . '!';
                    $cur_code = -10303;
                }
            }
            else {
                $cur_code = -10302;
                $cur_msg  = 'SOAP error: ' . $soap_error;
            }
        }
        else {
            if ($svc_loc ne '') {
                $cur_msg  = 'Service location not specified in configuration!';
                $cur_code = -10300;
            }
            if ($svc_ns ne '') {
                $cur_msg  = 'Service namespace not specified in configuration!';
                $cur_code = -10301;
            }
        }
        AOSICFG::write_ws_log("|> $this_fun: generating output") if $config::aosi_debug_level > 2;
        $code += abs($cur_code);
		    $msg .= '<plugin_result name="'.$svc.'" code="'.$cur_code.'">'. $cur_msg . '</plugin_result>';
    }
    $code = -$code;
    AOSICFG::write_ws_log("|> $this_fun(out): code=$code;msg=$msg") if $config::aosi_debug_level > 1;
    return $code, $msg;
}

1;
